﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.ComponentModel;
using System.Globalization;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;

namespace HeuristicLab.Problems.Instances.QAPGenerator.Views {
  [View("QAP Instance Generator Descriptor View")]
  [Content(typeof(QAPInstanceGeneratorDescriptor), IsDefaultView = true)]
  public partial class QAPInstanceGeneratorDescriptorView : AsynchronousContentView {
    protected bool SuppressEvents { get; set; }

    public new QAPInstanceGeneratorDescriptor Content {
      get { return (QAPInstanceGeneratorDescriptor)base.Content; }
      set { base.Content = value; }
    }

    public QAPInstanceGeneratorDescriptorView() {
      InitializeComponent();
    }

    protected override void DeregisterContentEvents() {
      Content.PropertyChanged -= ContentOnPropertyChanged;
      base.DeregisterContentEvents();
    }

    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.PropertyChanged += ContentOnPropertyChanged;
    }

    #region Event Handlers (Content)
    protected virtual void ContentOnPropertyChanged(object sender, PropertyChangedEventArgs eventArgs) {
      SuppressEvents = true;
      try {
        switch (eventArgs.PropertyName) {
          case "N":
            sizeNumericUpDown.Value = Content.N;
            break;
          case "Seed":
            seedTextBox.Text = Content.Seed.ToString(CultureInfo.CurrentCulture.NumberFormat);
            break;
        }
      } finally { SuppressEvents = false; }
    }
    #endregion

    protected override void OnContentChanged() {
      base.OnContentChanged();
      SuppressEvents = true;
      try {
        if (Content == null) {
          sizeNumericUpDown.Value = 0;
          seedTextBox.Text = string.Empty;
        } else {
          sizeNumericUpDown.Value = Content.N;
          seedTextBox.Text = Content.Seed.ToString(CultureInfo.CurrentCulture.NumberFormat);
        }
      } finally { SuppressEvents = false; }
    }


    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      sizeNumericUpDown.Enabled = !Locked && !ReadOnly && Content != null;
      seedTextBox.Enabled = !Locked && !ReadOnly && Content != null;
    }

    #region Event Handlers (child controls)
    protected virtual void sizeNumericUpDown_ValueChanged(object sender, EventArgs e) {
      if (SuppressEvents) return;
      Content.N = (int)sizeNumericUpDown.Value;
    }

    protected virtual void seedTextBox_Validating(object sender, CancelEventArgs e) {
      if (SuppressEvents) return;
      int seed;
      e.Cancel = !int.TryParse(seedTextBox.Text, NumberStyles.Integer, CultureInfo.CurrentCulture.NumberFormat, out seed) || seed < 0;
      if (e.Cancel) errorProvider.SetError(seedTextBox, "The given seed is not valid, enter a positive number smaller than 2^31.");
      else errorProvider.SetError(seedTextBox, String.Empty);
    }

    protected virtual void seedTextBox_Validated(object sender, EventArgs e) {
      if (SuppressEvents) return;
      var seed = int.Parse(seedTextBox.Text);
      Content.Seed = seed;
    }
    #endregion
  }
}
